SystemOrganization addCategory: #'Network-URI'!

Object subclass: #URI
	instanceVariableNames: 'fragment scheme schemeSpecificPart'
	classVariableNames: 'ClientClasses'
	poolDictionaries: ''
	category: 'Network-URI'!

!URI commentStamp: 'mir 2/20/2002 15:17' prior: 0!
A Uniform Resource Identifier (URI) is a compact string of characters for identifying an abstract or physical resource.
This implementation is based on http://www.ietf.org/rfc/rfc2396.txt.

!

URI subclass: #HierarchicalURI
	instanceVariableNames: 'authority query pathComponents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Network-URI'!

HierarchicalURI subclass: #DirectoryURI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Network-URI'!

!HierarchicalURI methodsFor: 'private' stamp: 'mir 6/20/2005 18:49'!
absoluteFromString: aString scheme: schemeName
	| remainder |
	super absoluteFromString: aString scheme: schemeName.

	"We now have the interesting part in schemeSpecficPart and can parse it further"

	"This check is somewhat redundant, just in case somebody calls this directly."
	remainder := schemeSpecificPart.
	(remainder isEmpty
		or: [remainder first ~~ $/])
		ifTrue: [(IllegalURIException new uriString: remainder) signal: 'Invalid absolute URI'].

	(aString beginsWith: '//')
		ifTrue: [remainder := self extractAuthority: (remainder copyFrom: 3 to: remainder size)].

	self extractSchemeSpecificPartAndFragment: remainder! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'mir 3/6/2002 14:46'!
absolutePath
	^self schemeSpecificPart isEmpty
		ifTrue: ['/']
		ifFalse: [self schemeSpecificPart]! !

!HierarchicalURI methodsFor: 'directory operations' stamp: 'mir 3/8/2002 10:51'!
assureExistance
	! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'mir 2/25/2002 18:37'!
authority
	^authority! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'mir 3/22/2007 12:46'!
baseName
	"returns the last component stripped of its extension"

	| baseName i |
	baseName := self name.
	i := baseName findLast: [:c | c = $.].
	^i = 0
		ifTrue: [baseName]
		ifFalse: [baseName copyFrom: 1 to: i-1].
! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'marcus.denker 2/20/2009 16:29'!
baseNameUnescaped
	"returns the last component stripped of its extension"

	^self baseName unescapePercents! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'JMM 8/2/2007 11:54'!
baseNameWithExtension
	"returns the last component foo.bar"

	^self pathComponents last.
! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'marcus.denker 2/20/2009 16:29'!
baseNameWithExtensionUnescaped
	"returns the last component foo.bar as unescaped "

	^self pathComponents last unescapePercents.
! !

!HierarchicalURI methodsFor: 'private' stamp: 'mir 4/18/2007 22:10'!
buildAbsolutePath: pathParts
	^String streamContents: [:stream |
		stream nextPut: $/.
		pathParts
			do: [:pathPart | stream nextPutAll: pathPart]
			separatedBy: [stream nextPut: $/]]! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'JMM 5/8/2006 16:28'!
extension
	"This method assumes a $. as extension delimiter"

	| i leafName |
	self pathComponents ifEmpty: [^''].
	leafName := self pathComponents last.
	i := leafName findLast: [:c | c = $.].
	^i = 0
		ifTrue: ['']
		ifFalse: [leafName copyFrom: i + 1 to: leafName size].
! !

!HierarchicalURI methodsFor: 'private' stamp: 'mir 2/27/2002 12:46'!
extractAuthority: aString
	| endAuthorityIndex authorityString |
	endAuthorityIndex := (aString indexOf: $/ ) - 1.
	endAuthorityIndex < 0
		ifTrue: [endAuthorityIndex := aString size].
	authorityString := aString copyFrom: 1 to: endAuthorityIndex.
	authority := URIAuthority fromString: authorityString.
	^aString copyFrom: endAuthorityIndex+1 to: aString size! !

!HierarchicalURI methodsFor: 'private' stamp: 'mir 2/26/2002 14:13'!
extractQuery: remainder
	| queryIndex |
	queryIndex := remainder indexOf: $?.
	queryIndex > 0
		ifFalse: [^remainder].
	query := remainder copyFrom: queryIndex to: remainder size.
	^remainder copyFrom: 1 to: queryIndex-1! !

!HierarchicalURI methodsFor: 'private' stamp: 'mir 2/26/2002 14:13'!
extractSchemeSpecificPartAndFragment: remainder
	super extractSchemeSpecificPartAndFragment: remainder.
	schemeSpecificPart := self extractQuery: schemeSpecificPart! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'mir 2/20/2002 17:26'!
host
	^self authority host! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'mir 4/18/2007 21:52'!
name
	"returns the last component"

	^self pathComponents last unescapePercents! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'mir 2/27/2002 14:21'!
path
"	^self schemeSpecificPart isEmpty
		ifTrue: ['/']
		ifFalse: [self schemeSpecificPart]"
	^self schemeSpecificPart! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'mir 3/17/2007 18:28'!
pathAndQuery
	^query
		ifNil: [self path]
		ifNotNil: [self path , self query]! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'mir 4/18/2007 22:12'!
pathComponents
	^pathComponents ifNil: [pathComponents := (self path findTokens: $/) collect: [:each | each unescapePercents]]! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'JMM 8/2/2007 11:50'!
pathComponentsUnescaped
	^(self path findTokens: $/) collect: [:e | e unescapePercents].! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'mir 2/20/2002 17:26'!
port
	^self authority port! !

!HierarchicalURI methodsFor: 'printing' stamp: 'mir 2/27/2002 12:51'!
printSchemeSpecificPartOn: stream
	self isAbsolute
		ifTrue: [stream nextPutAll: '//'].
	authority
		ifNotNil: [self authority printOn: stream].
	super printSchemeSpecificPartOn: stream.
	query
		ifNotNil: [stream nextPutAll: query]! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'mir 2/20/2002 17:27'!
query
	^query! !

!HierarchicalURI methodsFor: 'private' stamp: 'mir 2/27/2002 14:16'!
relativeFromString: aString
	| remainder authorityEnd |
	remainder := (aString beginsWith: '//')
		ifTrue: [
			authorityEnd := aString indexOf: $/ startingAt: 3.
			authorityEnd == 0
				ifTrue: [authorityEnd := aString size+1].
			self extractAuthority: (aString copyFrom: 3 to: authorityEnd-1)]
		ifFalse: [aString].
	self extractSchemeSpecificPartAndFragment: remainder! !

!HierarchicalURI methodsFor: 'private' stamp: 'mir 4/18/2007 22:11'!
removeComponentDotDotPairs: pathParts
	| dotDotIndex |
	dotDotIndex := pathParts indexOf: '..'.
	[dotDotIndex > 1]
		whileTrue: [
			pathParts
				removeAt: dotDotIndex;
				removeAt: dotDotIndex-1.
			dotDotIndex := pathParts indexOf: '..']! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'jannik.laval 2/5/2010 21:45'!
resolveRelativeURI: aURI
	| relativeURI newAuthority newPath pathParts newURI relComps |
	relativeURI := aURI asURI.

	relativeURI isAbsolute
		ifTrue: [^relativeURI].

	relativeURI authority
		ifNil: [
			newAuthority := self authority.
			(relativeURI path beginsWith: '/')
				ifTrue: [newPath := relativeURI path]
				ifFalse: [
					pathParts := (self path copyUpToLast: $/) findTokens: $/.
					relComps := relativeURI pathComponents.
					relComps removeAllSuchThat: [:each | each = '.'].
					pathParts addAll: relComps.
					pathParts removeAllSuchThat: [:each | each = '.'].
					self removeComponentDotDotPairs: pathParts.
					newPath := self buildAbsolutePath: pathParts.
					((relComps isEmpty
						or: [relativeURI path last == $/ 
						or: [(relativeURI path endsWith: '/..')
						or: [relativeURI path = '..'
						or: [relativeURI path endsWith: '/.' ]]]])
						and: [newPath size > 1])
						ifTrue: [newPath := newPath , '/']]]
		ifNotNil: [
			newAuthority := relativeURI authority.
			newPath := relativeURI path].

	newURI := String streamContents: [:stream |
		stream nextPutAll: self scheme.
		stream nextPut: $: .
		newAuthority notNil
			ifTrue: [
				stream nextPutAll: '//'.
				newAuthority printOn: stream].
		newPath notNil
			ifTrue: [stream nextPutAll: newPath].
		relativeURI query notNil
			ifTrue: [stream nextPutAll: relativeURI query].
		relativeURI fragment notNil
			ifTrue: [
				stream nextPut: $# .
				stream nextPutAll: relativeURI fragment]].
	^newURI asURI! !

!HierarchicalURI methodsFor: 'accessing' stamp: 'mir 2/20/2002 17:27'!
userInfo
	^self authority userInfo! !

URI subclass: #OpaqueURI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Network-URI'!

!OpaqueURI methodsFor: 'testing' stamp: 'mir 2/20/2002 16:55'!
isOpaque
	^true! !

!URI class methodsFor: 'private' stamp: 'mir 4/15/2007 21:26'!
absoluteFromString: aString scheme: scheme
	| remainder |
	remainder := aString copyFrom: scheme size+2 to: aString size.
	remainder isEmpty
		ifTrue: [(IllegalURIException new uriString: aString) signal: 'Invalid absolute URI'].
	^(remainder first = $/
		ifTrue: [(aString last = $/
					ifTrue: [DirectoryURI]
					ifFalse: [HierarchicalURI])]
		ifFalse: [OpaqueURI]) new absoluteFromString: remainder scheme: scheme! !

!URI class methodsFor: 'private' stamp: 'mir 2/20/2002 17:23'!
extractSchemeFrom: aString
	| colonIndex slashIndex |
	colonIndex := aString indexOf: $: .
	^colonIndex > 0
		ifTrue: [
			slashIndex := aString indexOf: $/ .
			(slashIndex == 0
				or: [colonIndex < slashIndex])
				ifTrue: [aString copyFrom: 1 to: colonIndex-1]
				ifFalse: [nil]]
		ifFalse: [nil]! !

!URI class methodsFor: 'instance creation' stamp: 'mir 4/15/2007 21:22'!
fromString: aString
	| parseString scheme |
	parseString := aString withBlanksTrimmed.
	scheme := self extractSchemeFrom: parseString.
	^scheme
		ifNil: [aString size > 1
				ifTrue: [(aString last = $/
					ifTrue: [DirectoryURI]
					ifFalse: [HierarchicalURI]) new relativeFromString: aString]
				ifFalse: [HierarchicalURI new relativeFromString: aString]]
		ifNotNil: [self absoluteFromString: aString scheme: scheme]
! !

!URI class methodsFor: 'initialization' stamp: 'mir 3/1/2002 15:18'!
initialize
	"URI initialize"

	ClientClasses := Dictionary new.
	ClientClasses
		at: 'http' put: #HTTPClient;
		at: 'ftp' put: #FTPClient;
		at: 'file' put: #FileDirectory
! !

!URI methodsFor: 'testing' stamp: 'bf 1/26/2004 14:40'!
= otherURI
	^ self class = otherURI class
		and: [self asString = otherURI asString]! !

!URI methodsFor: 'private' stamp: 'mir 2/20/2002 17:18'!
absoluteFromString: remainder scheme: schemeName
	scheme := schemeName.
	self extractSchemeSpecificPartAndFragment: remainder! !

!URI methodsFor: 'converting' stamp: 'mir 3/6/2002 14:54'!
asText
	^self asString asText! !

!URI methodsFor: 'converting' stamp: 'mir 2/26/2002 15:15'!
asURI
	^self! !

!URI methodsFor: 'converting' stamp: 'tb 5/24/2006 22:04'!
asUrl

	^self asString asUrl! !

!URI methodsFor: 'private' stamp: 'lr 3/14/2010 21:13'!
clientClass
	^ Smalltalk globals at: (ClientClasses at: self scheme ifAbsent: [ ClientClasses at: 'file' ])! !

!URI methodsFor: 'retrieval' stamp: 'mir 10/20/2003 16:31'!
contentStream
	^self clientClass contentStreamForURI: self! !

!URI methodsFor: 'retrieval' stamp: 'JMM 7/20/2006 00:51'!
contentUTF8Stream
	^self clientClass contentUTF8StreamForURI: self! !

!URI methodsFor: 'retrieval' stamp: 'JMM 7/20/2006 00:52'!
contentUTF8WriteableStream
	^self clientClass contentUTF8WriteableStreamForURI: self! !

!URI methodsFor: 'private' stamp: 'mir 2/27/2002 14:18'!
extractSchemeSpecificPartAndFragment: remainder
	| fragmentIndex |
	fragmentIndex := remainder indexOf: $# .
	fragmentIndex > 0
		ifTrue: [
			schemeSpecificPart := remainder copyFrom: 1 to: fragmentIndex-1.
			fragment := remainder copyFrom: fragmentIndex+1 to: remainder size]
		ifFalse: [schemeSpecificPart := remainder]! !

!URI methodsFor: 'accessing' stamp: 'mir 2/20/2002 17:26'!
fragment
	^fragment! !

!URI methodsFor: 'testing' stamp: 'bf 1/26/2004 14:40'!
hash
	^ self asString hash! !

!URI methodsFor: 'testing' stamp: 'mir 2/20/2002 16:55'!
isAbsolute
	^self scheme notNil! !

!URI methodsFor: 'testing' stamp: 'mir 2/20/2002 16:55'!
isOpaque
	^false! !

!URI methodsFor: 'testing' stamp: 'mir 2/20/2002 16:55'!
isRelative
	^self isAbsolute not! !

!URI methodsFor: 'testing' stamp: 'JMM 7/13/2006 10:55'!
isURI
	^true! !

!URI methodsFor: 'accessing' stamp: 'mir 6/10/2007 19:00'!
mimeType
	^MIMEType forExtension: self extension! !

!URI methodsFor: 'stream creation' stamp: 'mir 4/16/2007 21:36'!
openStream: readWrite
	^self openStream: readWrite forceNew: false! !

!URI methodsFor: 'stream creation' stamp: 'mir 4/16/2007 18:41'!
openStream: readWrite forceNew: forceNew
	^self openStream: readWrite forceNew: forceNew encoding: nil! !

!URI methodsFor: 'stream creation' stamp: 'mir 6/10/2007 18:38'!
openStream: readWrite forceNew: forceNew encoding: stringEncoding
	^self directory openStream: self name mode: readWrite forceNew: forceNew encoding: stringEncoding! !

!URI methodsFor: 'printing' stamp: 'mir 2/26/2002 14:56'!
printOn: stream
	self isAbsolute
		ifTrue: [
			stream nextPutAll: self scheme.
			stream nextPut: $: ].
	self printSchemeSpecificPartOn: stream.
	fragment
		ifNotNil: [
			stream nextPut: $# .
			stream nextPutAll: self fragment]
! !

!URI methodsFor: 'printing' stamp: 'mir 2/26/2002 14:55'!
printSchemeSpecificPartOn: stream
	stream nextPutAll: self schemeSpecificPart! !

!URI methodsFor: 'stream creation' stamp: 'mir 4/16/2007 15:28'!
readStream
	^self openStream: #read! !

!URI methodsFor: 'stream creation' stamp: 'mir 4/16/2007 15:29'!
readWriteStream
	^self openStream: #readWrite! !

!URI methodsFor: 'accessing' stamp: 'mir 2/26/2002 19:17'!
resolveRelativeURI: relativeURI
	self shouldNotImplement! !

!URI methodsFor: 'content retrieval' stamp: 'mir 4/16/2007 18:56'!
retrieveContentsEncoding: stringEncoding
	^self openStream: #read forceNew: false encoding: stringEncoding! !

!URI methodsFor: 'accessing' stamp: 'mir 2/20/2002 16:53'!
scheme
	^scheme! !

!URI methodsFor: 'accessing' stamp: 'JMM 2/16/2008 11:22'!
schemeIsFile
	self scheme ifNil: [^false].
	^self scheme asLowercase = 'file'! !

!URI methodsFor: 'accessing' stamp: 'JMM 2/16/2008 11:22'!
schemeIsHttp
	self scheme ifNil: [^false].
	^self scheme asLowercase = 'http'! !

!URI methodsFor: 'private' stamp: 'mir 2/25/2002 16:10'!
schemeSpecificPart
	^schemeSpecificPart! !

!URI methodsFor: 'stream creation' stamp: 'mir 4/16/2007 15:29'!
writeStream
	^self openStream: #write! !

Object subclass: #URIAuthority
	instanceVariableNames: 'host port userInfo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Network-URI'!

!URIAuthority class methodsFor: 'instance creation' stamp: ''!
fromString: authorityString
	^self new fromString: authorityString! !

!URIAuthority methodsFor: 'private' stamp: 'mir 2/25/2002 19:04'!
fromString: authorityString
	| userInfoEnd remainder hostEnd |
	userInfoEnd := authorityString indexOf: $@.
	remainder := userInfoEnd > 0
		ifTrue: [
			userInfo := authorityString copyFrom: 1 to: userInfoEnd-1.
			authorityString copyFrom: userInfoEnd+1 to: authorityString size]
		ifFalse: [authorityString].
	hostEnd := remainder indexOf: $: .
	hostEnd > 0
		ifTrue: [
			host := remainder copyFrom: 1 to: hostEnd-1.
			port := (remainder copyFrom: hostEnd+1 to: remainder size) asNumber]
		ifFalse: [host := remainder]! !

!URIAuthority methodsFor: 'accessing' stamp: 'mir 2/20/2002 17:27'!
host
	^host! !

!URIAuthority methodsFor: 'accessing' stamp: 'mir 2/20/2002 17:27'!
port
	^port! !

!URIAuthority methodsFor: 'printing' stamp: 'mir 2/26/2002 14:52'!
printOn: stream
	userInfo
		ifNotNil: [
			stream nextPut: $@ .
			stream nextPutAll: userInfo].
	stream nextPutAll: host.
	port
		ifNotNil: [
			stream nextPut: $: .
			port printOn: stream] ! !

!URIAuthority methodsFor: 'accessing' stamp: 'mir 2/20/2002 17:28'!
userInfo
	^userInfo! !

!MacFileDirectory class methodsFor: '*network-uri' stamp: 'nice 1/5/2010 15:59'!
privateFullPathForURI: aURI
	| path |

	path := String streamContents: [ :s | | first |
		first := false.
		aURI pathComponents do: [ :p |
			first ifTrue: [ s nextPut: self pathNameDelimiter ].
			first := true.
			s nextPutAll: p ] ].
	aURI path last = $/ ifTrue: [path := path,FileDirectory slash].
	^path unescapePercents
! !

!FileStream class methodsFor: '*network-uri' stamp: 'JMM 2/3/2008 14:17'!
oldFileFullyNamed: t1 
	^ self concreteStream
		oldFileNamed: t1! !

!FileStream class methodsFor: '*network-uri' stamp: 'JMM 2/3/2008 14:14'!
readOnlyFileFullyNamed: t1 
	^ self concreteStream
		readOnlyFileFullyNamed: t1! !

!FileStream methodsFor: '*network-uri' stamp: 'bf 1/27/2006 18:01'!
uri
	^self directory uri resolveRelativeURI: self localName encodeForHTTP! !

!FileStream methodsFor: '*network-uri' stamp: 'fbs 2/2/2005 13:23'!
url
	"Convert my path into a file:// type url String."
	
	^self asUrl asString! !

!String methodsFor: '*network-uri' stamp: 'mir 2/26/2002 14:59'!
asURI
	"convert to a Url"
	"'http://www.cc.gatech.edu/' asURI"
	"'msw://chaos.resnet.gatech.edu:9000/' asURI"
	^URI fromString: self! !

!String methodsFor: '*network-uri' stamp: 'KR 9/22/2005 23:06'!
unescapePercents
	"decode %xx form.  This is the opposite of #encodeForHTTP"
	^ self unescapePercentsWithTextEncoding: 'utf-8'.! !

!String methodsFor: '*network-uri' stamp: 'ky 7/8/2006 17:56'!
unescapePercentsWithTextEncoding: encodingName 
	"decode string including %XX form"
	| unescaped char asciiVal specialChars oldPos pos converter |
	unescaped := ReadWriteStream on: String new.
	specialChars := '+%' asCharacterSet.
	oldPos := 1.
	[pos := self indexOfAnyOf: specialChars startingAt: oldPos.
	pos > 0]
		whileTrue: [unescaped
				nextPutAll: (self copyFrom: oldPos to: pos - 1).
			char := self at: pos.
			(char = $%
					and: [pos + 2 <= self size])
				ifTrue: [asciiVal := (self at: pos + 1) asUppercase digitValue * 16 + (self at: pos + 2) asUppercase digitValue.
					asciiVal > 255
						ifTrue: [^ self].
					unescaped
						nextPut: (Character value: asciiVal).
					pos := pos + 3.
					pos <= self size
						ifFalse: [char := nil].
					oldPos := pos]
				ifFalse: [char = $+
						ifTrue: [unescaped nextPut: Character space]
						ifFalse: [unescaped nextPut: char].
					oldPos := pos + 1]].
	oldPos <= self size
		ifTrue: [unescaped
				nextPutAll: (self copyFrom: oldPos to: self size)].
	converter := (TextConverter newForEncoding: encodingName)
				ifNil: [TextConverter newForEncoding: nil].
	^ [unescaped contents convertFromWithConverter: converter]
		on: Error
		do: ["the contents may be squeak-encoded"
			unescaped contents]! !

!FileDirectory class methodsFor: '*network-uri' stamp: 'JMM 2/3/2008 14:17'!
contentStreamForURI: aURI
	| fullPath stream |

	fullPath := self fullPathForURI: aURI.
	stream := FileStream readOnlyFileFullyNamed: fullPath.
	^stream binary
! !

!FileDirectory class methodsFor: '*network-uri' stamp: 'JMM 2/3/2008 14:14'!
contentUTF8StreamForURI: aURI
	| fullPath |
	fullPath := self fullPathForURI: aURI.
	^FileStream readOnlyFileFullyNamed: fullPath
! !

!FileDirectory class methodsFor: '*network-uri' stamp: 'JMM 2/3/2008 14:17'!
contentUTF8WriteableStreamForURI: aURI
	| fullPath |
	fullPath := self fullPathForURI: aURI.
	^FileStream oldFileFullyNamed: fullPath! !

!FileDirectory class methodsFor: '*network-uri' stamp: 'JMM 2/3/2008 14:17'!
contentWriteableStreamForURI: aURI
	| fullPath stream |
	fullPath := self fullPathForURI: aURI.
	stream := FileStream oldFileFullyNamed: fullPath.
	^stream binary! !

!FileDirectory class methodsFor: '*network-uri' stamp: 'mir 6/20/2005 18:53'!
fullPathForURI: aURI
	^self activeDirectoryClass privateFullPathForURI: (FileDirectory default uri resolveRelativeURI: aURI)! !

!FileDirectory class methodsFor: '*network-uri' stamp: 'mir 6/20/2005 18:53'!
privateFullPathForURI: aURI
	^(aURI path copyReplaceAll: '/' with: self slash) unescapePercents! !

!FileDirectory class methodsFor: '*network-uri' stamp: 'mir 6/20/2005 18:34'!
uri: aURI
	^self on: (FileDirectory fullPathForURI: aURI)! !

!FileDirectory methodsFor: '*network-uri' stamp: 'bf 1/27/2006 18:00'!
uri
	"Convert my path into a file:// type url. For odd characters use %20 notation."

	| list |
	list := self pathParts.
	^(String streamContents: [:strm |
		strm nextPutAll: 'file:'.
		list do: [:each | strm nextPut: $/; nextPutAll: each encodeForHTTP].
		strm nextPut: $/]) asURI! !

!AcornFileDirectory classSide methodsFor: '*network-uri' stamp: 'nice 1/5/2010 15:59'!
privateFullPathForURI: aURI
	"derive the full filepath from aURI"
	| path |

	path := String streamContents: [ :s | | first |
		first := false.
		aURI pathComponents do: [ :p |
			first ifTrue: [ s nextPut: self pathNameDelimiter ].
			first := true.
			s nextPutAll: p ] ].
	^path unescapePercents
! !

!StandardFileStream class methodsFor: '*network-uri' stamp: 'JMM 2/3/2008 14:16'!
oldFileFullyNamed: t2 

	^ (self isAFileNamed: t2)
		ifTrue: [self new open: t2 forWrite: true]
		ifFalse: [(FileDoesNotExistException fileName: t2) signal]! !

!StandardFileStream class methodsFor: '*network-uri' stamp: 'JMM 2/3/2008 14:12'!
readOnlyFileFullyNamed: t1 
	| t3 |
	t3 := self new open: t1 forWrite: false.
	^ t3 isNil
		ifTrue: [((FileDoesNotExistException fileName: t1)
				readOnly: true) signal]
		ifFalse: [t3]! !

!DosFileDirectory class methodsFor: '*network-uri' stamp: 'pg 1/29/2006 15:37'!
privateFullPathForURI: aURI
	| path |
	path := aURI path unescapePercents.

	"Check for drive notation (a: etc)"
	path size > 1
		ifTrue: [
			((path at: 3) = $:)
				ifTrue: [path := path copyFrom: 2 to: path size]
				ifFalse: [
					"All other cases should be network path names (\\xxx\sdsd etc)"
					path := '/' , path]].

	^path copyReplaceAll: '/' with: self slash! !

Error subclass: #IllegalURIException
	instanceVariableNames: 'uriString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Network-URI'!

!IllegalURIException methodsFor: 'accessing' stamp: 'mir 2/20/2002 17:21'!
uriString
	^uriString! !

!IllegalURIException methodsFor: 'accessing' stamp: 'mir 2/20/2002 17:20'!
uriString: aString
	uriString := aString! !
URI initialize!
